# -*- coding: utf-8 -*-

import cStringIO
import csv
from decimal import Decimal
import datetime
from neptuno.util import default_fmt_float, datetostr, timetostr

class FicheroCSV(object):
    
    date_fmt = '%d/%m/%Y'
    time_fmt = '%H:%M'
    true_const = 't'
    false_const = 'f'    
    
    def __init__(self):
        # crear fichero CSV en memoria
        self.fichero_csv = cStringIO.StringIO()
    
        # crear writer CSV
        self.w_csv = csv.writer(self.fichero_csv, delimiter=';', 
                                quoting=csv.QUOTE_ALL)
        
        self.float_fmt = default_fmt_float
        
    def add(self, row):
        """
        Añade una fila al fichero CSV.
        
        IN
          row <list>
        """
        
        float_fmt = self.float_fmt
        
        row_resultado = []
        for i in row:
            
            if isinstance(i, float) or isinstance(i, Decimal):
                row_resultado.append(float_fmt(i))
                
            elif isinstance(i, bool):
                row_resultado.append((self.true_const if i else self.false_const))                 

            elif isinstance(i, int):
                row_resultado.append('%d' % i)

            elif isinstance(i, datetime.datetime):
                datetime_fmt = '%s %s' % (self.date_fmt, self.time_fmt)
                row_resultado.append(i.strftime(datetime_fmt))

            elif isinstance(i, datetime.date):                
                row_resultado.append(datetostr(i, fmt=self.date_fmt))
                
            elif isinstance(i, datetime.time):
                row_resultado.append(timetostr(i, fmt=self.time_fmt))
                
            elif isinstance(i, unicode):
                row_resultado.append(i.encode('utf-8'))

            else:
                row_resultado.append(str(i or '')) #.decode('utf-8'))
                
        self.w_csv.writerow(row_resultado)
    
    def read(self, encoding=None):
        """
        Devuelve el contenido del fichero CSV en forma de cadena codificado
        según el parámetro 'encoding' (por defecto 'iso-8859-1')
        
        IN
          encoding <str> => 'iso-8859-1'
          
        OUT
          <str>
        """
        
        if not encoding:
            encoding = 'iso-8859-1'
        
        self.fichero_csv.seek(0)
        resultado = self.fichero_csv.read().decode('utf-8').encode(encoding)
        #resultado = self.fichero_csv.read()
        
        return resultado
    
    def __del__(self):
        self.fichero_csv.close()


# -*- coding: utf-8 -*-

import re
import simplejson
import datetime as dt
from decimal import Decimal

def get_parametros(req, parametros):
    form = req.form
    
    resultado = {}
    for p in parametros:
        valor = form.get(p, None)
        if valor != None:
            valor = valor.value
            
        resultado[p] = valor
            
    return resultado

class EFaltaParametro(Exception):
    
    def __init__(self, param):
        Exception.__init__(self)
        self.param = param
        
    def __str__(self):
        return 'Falta el parámetro "%s"' % self.param

def get_param(form, nombre, tipo, opcional=False, por_defecto=None):
    """
    Devuelve el parámetro 'nombre' de tipo 'tipo' del diccionario 'form'
    
    Ejemplos:
      # un valor entero "obligatorio"
      i = get_param(req.form, 'entero', int)
      
      # un valor entero "opcional" (valor por defecto=None)
      i = get_param(req.form, 'entero', int, opcional=True)
      
      # un valor entero "opcional" con valor por defecto
      i = get_param(req.form, 'entero', int, opcional=True, por_defecto=100)
      
      # un valor que se obtiene después de pasar por una función
      x = get_param(req.form, 'x', 
                    lambda v: simplejson.loads(str(v)))
                    
      f = get_param(req.form, 'fecha', strtodate)     
    """
    
    if not opcional:
        if not form.has_key(nombre):
            raise EFaltaParametro(nombre)
        
        else:
            try:
                return tipo(form[nombre].value)
            except:
                return por_defecto
                    
    else:
        if form.has_key(nombre):
            try:
                return tipo(form[nombre].value)
            except:
                return por_defecto
        
        else:
            return por_defecto

def get_paramw(params, nombre, tipo, opcional=False, por_defecto=None):
    """
    Devuelve el parámetro 'nombre' de tipo 'tipo' del diccionario 'params'
    
    Ejemplos:
      # un valor entero "obligatorio"
      i = get_paramw(params, 'entero', int)
      
      # un valor entero "opcional" (valor por defecto=None)
      i = get_paramw(params, 'entero', int, opcional=True)
      
      # un valor entero "opcional" con valor por defecto
      i = get_paramw(params, 'entero', int, opcional=True, por_defecto=100)
      
      # un valor que se obtiene después de pasar por una función
      x = get_paramw(params, 'x', 
                     lambda v: simplejson.loads(str(v)))
                    
      f = get_paramq(params, 'fecha', strtodate)     
    """
    
    if not opcional:
        if not params.has_key(nombre):
            raise EFaltaParametro(nombre)
        
        else:
            try:
                return tipo(params[nombre])
            except:
                return por_defecto
                    
    else:
        if params.has_key(nombre):
            try:
                return tipo(params[nombre])
            except:
                return por_defecto
        
        else:
            return por_defecto
        
def jsonize(func):
    def _jsonize(_self, *args, **kwargs):
        return simplejson.dumps(func(_self, *args, **kwargs))
        
    return _jsonize

def get_params(form, excepciones):
    """
    Devuelve un diccionario con los parámetros de 'form' que no están
    incluidos en 'excepciones'
    
    IN
      form         <dict>
      excepciones  <list>
      
    OUT
      <dict>
    """
    
    resultado = {}
    for k, v in form.items():
        if k not in excepciones:
            resultado[k] = str(v.value)
        
    return resultado

def get_paramsw(params, excepciones):
    """
    Devuelve un diccionario con los parámetros de 'params' que no están
    incluidos en 'excepciones'.
    
    IN
      params       <dict>
      excepciones  <list>
      
    OUT
      <dict>
    """
    
    resultado = {}
    for k, v in params.iteritems():
        if k not in excepciones:
            resultado[k] = str(v)
        
    return resultado

def get_usuario(form):    
    return get_param(form, 'id_usuario', int), get_param(form, 'id_sesion', str),

#def strtodate(s, fmt='%d/%m/%Y'):
#    return datetime.strptime(s, fmt).date()

class EFechaIncorrecta(Exception):
    def __init__(self, s):
        Exception.__init__(self)
        self.s = s
        
    def __str__(self):
        return 'La fecha "%s" es incorrecta' % self.s

def strtodate(s, fmt='%d/%m/%Y', no_exc=False):
    """
    IN
      s <str>
      
    OUT
      <date>
    """
    
    # build regex
    regex = r'^\s*%s\s*$' % (fmt.replace('%d', r'(?P<day>\d{1,2})').\
                             replace('%m', r'(?P<month>\d{1,2})').\
                             replace('%Y', r'(?P<year>\d{4})'))
        
    m1 = re.search(regex, s)
    if m1:
        try:
            day = int(m1.groupdict().get('day') or 1)
            month = int(m1.groupdict().get('month') or 1)
            year = int(m1.groupdict().get('year') or 1)
            
            return dt.date(year, month, day)
        
        except Exception:
            if no_exc: return None
            raise Exception('Fecha incorrecta: "%s"' % s)

#def strtodate(s, no_exc=False, month_day=False):
#    """
#    Devuelve la fecha representada por la cadena 's'.
#    Formatos posibles para 's':
#      d/m/yyyy (1/2/1970, 01/2/1970, 1/02/1970, 01/02/1970)
#      d-m-yyyy (1-2-1970, 01-2-1970, 1-02-1970, 01-02-1970)
#      yyyy/m/d (1970/2/1, 1970/2/01, 1970/02/1, 1970/02/01)
#      yyyy-m-d (1970-2-1, 1970-2-01, 1970-02-1, 1970-02-01)
#      
#      *(d y m representados con 1 ó 2 dos dígitos)     
#    
#    IN
#      s <str>
#      
#    OUT
#      <date>
#    """
#    
#    # dd/mm/yyyy
#    # dd-mm-yyyy
#    m1 = re.search(r'^\s*(\d{1,2})[/-](\d{1,2})[/-](\d{4})\s*$', s)
#    if m1:
#        try:
#            return date(int(m1.group(3)),
#                        int(m1.group(2)),
#                        int(m1.group(1))
#                        )
#        except:
#            if no_exc: return None
#            raise EFechaIncorrecta(s)
#        
#    else:
#        # yyyy/mm/dd
#        # yyyy-mm-dd
#        m2 = re.search(r'^\s*(\d{4})[/-](\d{1,2})[/-](\d{1,2})\s*$', s)
#        
#        if m2:
#            try:        
#                return date(int(m2.group(1)),
#                            int(m2.group(2)),
#                            int(m2.group(3))
#                            )
#            except:
#                if no_exc: return None
#                raise EFechaIncorrecta(s)
#        else:
#            if no_exc: return None
#            raise EFechaIncorrecta(s)
        
def strtodate2(s):
    return strtodate(s, no_exc=True)

def strtotime(s):
    m = re.search(r'^(\d{1,2}):(\d{1,2}):?(\d{1,2})?', s)
    return dt.time(int(m.group(1)), int(m.group(2)), int(m.group(3) or 0))

def strtobool(s):
    if s.lower() == 'true':
        return True
    elif s.lower() == 'false':
        return False
    else:
        raise TypeError('No es un tipo booleano')
    
def strlen(v, l):
    """
    """
    
    v = str(v)
    
    if len(v) != l:
        raise TypeError('Longitud incorrecta')
    
    return v
    
# ...tostr
def datetostr(d, fmt='%d/%m/%Y'):
    if d is None:
        return ''
    
    else:
        try:
            return d.strftime(fmt)
        except ValueError:
            return ''

def timetostr(t, fmt='%H:%M'):
    if t is None:
        return ''
    
    else:
        return t.strftime(fmt)
    
def inicio_fin_mes(d):
    """
    Devuelve el primer y el último día del mes en la fecha 'fecha'.
    
    IN
      fecha <date>
      
    OUT
      (<date>, <date>)
    """
    
    inicio = dt.date(d.year, d.month, 1)
    mes_siguiente = inicio + dt.timedelta(days=31)
    fin = dt.date(mes_siguiente.year, mes_siguiente.month, 1) - dt.timedelta(days=1)

    return (inicio, fin,)

def inicio_fin_semana(d):
    """
    Returns monday and sunday of the week for the given date. 
    IN
      d  <date>
      
    OUT
      (<date>, <date>,)
    """
    
    inicio = d - dt.timedelta(d.weekday())
    fin = inicio + dt.timedelta(days=6)
    
    return (inicio, fin,)
    
def format_float(value, thousands_sep=',', decimal_sep='.', 
                 show_sign=False, n_dec=2):
    """
    IN
      value          <float> / <str>
      thousands_sep  <str>  (opcional => ',')
      decimal_sep    <str>  (opcional => '.')
      show_sign      <bool> (opcional => False)
      n_dec          <int>  (opcional => 2)
      
    OUT
      <str>
      x,xxx.xx
      x.xxx,xx
      +x,xxx.xx
      -x,xxx.xx      
    """
    
    fmt = '%%+%d.%df' % (n_dec, n_dec)
    
    if isinstance(value, int):
        value = float(value)
    
    if isinstance(value, float) or isinstance(value, Decimal):
        text = fmt % value
        
    elif isinstance(value, str) or value is None:
        text = fmt % float(value or 0)
    
    m = re.search(r'([\+-]?)0*(\d*)\.(\d+)', text)
    
    sign = m.group(1)
    if sign and sign != '-':
        if not show_sign:
            sign = ''
    
    int_part = m.group(2)
    dec_part = m.group(3)
    
    if int_part:
        parte_entera = range(len(int_part))
        parte_entera.reverse()
        
        j = 0
        result = ''
        for i, j in zip(parte_entera, range(len(int_part))):
            if j != 0 and j % 3 == 0:
                result = thousands_sep + result
                
            result = int_part[i] + result
            
    else:
        result = '0'        
    
    # 1,234.56
    return '%(sign)s%(int_part)s%(sep)s%(dec_part)s' % \
                dict(sign=sign or '',
                     int_part=result, 
                     sep=decimal_sep, 
                     dec_part=dec_part)

def default_fmt_float(value):
    return format_float(value, thousands_sep='.', decimal_sep=',')


# -*- coding: utf-8 -*-

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from const_datos_neptuno import CONF_HOST, CONF_DB, CONF_USER, CONF_PASSW
from neptuno.const_datos_neptuno import IMPLTYPE_FIREBIRD, IMPLTYPE_POSTGRESQL

class Conexion(object):
    
    def __init__(self, config, impl_type=IMPLTYPE_POSTGRESQL):
        """
        IN
          config {
                  'host':     <str>,
                  'db':       <str>,
                  'user':     <str>,
                  'password': <str>
                 }
                 
          impl_type <str> ('postgres', 'firebird')
        """
        
        # config
        self.config = config            
        
        # impl_type
        self.impl_type = impl_type            
            
        self.engine = create_engine(self.connectionstr(self.config),
                                    pool_size=1, pool_recycle=30)
        
        Session = sessionmaker(bind=self.engine)
        self.session = Session()
        
    def connectionstr(self, config):
        """
        Implementaciones actuales:
          postgres://postgres:5390post@localhost/tandem
          firebird://SYSDBA:masterkey@localhost/argos        
        """
        tipo = 'postgresql'
        if self.impl_type == IMPLTYPE_FIREBIRD:
            tipo = 'firebird'                     

        return '%s://%s:%s@%s/%s' % \
            (tipo,
             config[CONF_USER], config[CONF_PASSW], 
             config[CONF_HOST], config[CONF_DB])

    def __del__(self):        
        self.session.close()        
        self.engine.dispose()


# -*- coding: utf-8 -*-

from datetime import datetime, date

# strto...
def strtodate(s, fmt='%d/%m/%Y'):
    return datetime.strptime(s, fmt).date()

def strtotime(s, fmt='%H:%M'):
    return datetime.strptime(s, fmt).time()

def strtobool(s):
    if s.lower() == 'true':
        return True
    elif s.lower() == 'false':
        return False    
    else:
        raise TypeError('No es un tipo booleano')    

# ...tostr
def datetostr(d, fmt='%d/%m/%Y'):
    if d is None:
        return ''
    
    else:
        try:
            return d.strftime(fmt)
        except ValueError:
            return ''

def timetostr(t, fmt='%H:%M'):
    if t is None:
        return ''
    
    else:
        return t.strftime(fmt)
    
def inicio_fin_mes(fecha):
    """
    Devuelve el primer y el último día del mes en la fecha 'fecha'.
    IN
      fecha <date>
      
    OUT
      (<date>, <date>)
    """
    
    inicio = date(fecha.year, fecha.month, 1)
    
    mes_siguiente = date.fromordinal(inicio.toordinal() + 31)
    fin = date.fromordinal(date(mes_siguiente.year, 
                                mes_siguiente.month, 
                                1).toordinal() - 1)
    
    return (inicio, fin,)


# -*- coding: utf-8 -*-

class Dict(dict):
    def __init__(self, **kw):
        for k, v in kw.iteritems():
            #print type(v)
            if isinstance(v, dict) or isinstance(v, Dict):
                self[k] = Dict(**v)
                
            elif isinstance(v, list):
                v2 = []
                for item in v:
                    #print type(item)
                    if isinstance(item, dict) or isinstance(item, Dict):
                        #print 'v =', v
                        v2.append(Dict(**item))
                        
                    else:
                        #print '*v=', v
                        v2.append(item)
                        
                self[k] = v2

            elif isinstance(v, tuple):
                v2 = None
                for item in v:
                    
                    if isinstance(v, dict) or isinstance(v, Dict):
                        if not v2:
                            v2 = (Dict(**item),)
                            
                        else:
                            v2 = v2 + (Dict(**item),)
                            
                    else:
                        if not v2:
                            v2 = (item,)
                            
                        else:
                            v2 = v2 + (item,)
                    
                self[k] = v2

            else:
                self[k] = v
            
    def __getattr__(self, name):
        if self.has_key(name):
            return self[name]
        
    def __setattr__(self, name, value):
        self[name] = value
            
            
if __name__ == '__main__':
    
    origen = {"cargos": [{"id": 1, "desc": "uno"}, {"id": 2, "desc": "dos"}],
              "tupla": ({"a": 1000}, {"a": 2000},)
              }
    
    destino = Dict(**origen)
    print destino.cargos
    print destino.cargos[0].id
    
    print destino.tupla
    print destino.tupla[0].a


# -*- coding: utf-8 -*-

from lxml import etree
from neptuno.util import strtodate, strtotime
from xlwt import Workbook, Formula
from xlwt.Style import easyxf, XFStyle
import re
import xlrd

__all__ = ['XLSReport']

class XLSReport(object):
    
    class Util(object):
        
        def __init__(self):
            self.n = 0
            self.marks = {}
            
        def init_count(self):
            self.n = 0
            self.marks = {}
            
        def current_line(self):
            return self.n
        
        def line_feed(self, n=1):
            self.n += n
            return self.n
        
        def add_bookmark(self, name):
            self.marks[name] = self.n
            return self.n
            
        def bookmark(self, name):
            return self.marks[name]

    def __init__(self, template):
        self.tmpl = template

    def calcular_estilo(self, estilo):
        
        style = []
        
        if estilo != None:
            # font
            fuente = estilo.find('font')
            if fuente != None:
                
                font = []
                
                # name
                if fuente.attrib.get('name', None):
                    font.append('name %s' % fuente.attrib['name'])
                    
                # size (height)
                if fuente.attrib.get('size', None):
                    font.append('height %d' % (int(fuente.attrib['size'])*25))
                    
                # color
                if fuente.attrib.get('color', None):
                    color = fuente.attrib['color']
                    font.append('color %s' % color)
                    
                # bold
                if fuente.attrib.get('bold', None):
                    font.append('bold %s' % fuente.attrib['bold'])
                
                # italic
                if fuente.attrib.get('italic', None):
                    font.append('italic %s' % fuente.attrib['italic'])
                    
                # underline
                if fuente.attrib.get('underline', None):
                    font.append('underline %s' % fuente.attrib['underline'])
                
                # añadir "font"
                if font:
                    #print font
                    style.append('font: %s' % (','.join(font)))
                    
            # align
            if estilo.find('align') != None:
                
                al = estilo.find('align')
                align = []
                
                # halign (horiz align)
                if al.attrib.get('hor', None):
                    align.append('horiz %s' % al.attrib['hor'])
                    
                # valign (vert align)
                if al.attrib.get('ver', None):
                    align.append('vert %s' % al.attrib['ver'])
                    
                if align:
                    style.append('align: %s' % (','.join(align)))
                    
            pattern = []
            
            # bgcolor
            patron_relleno = ''
            if estilo.attrib.get('pattern', None):
                patron_relleno = estilo.attrib['pattern']
                pattern.append('pattern %s' % patron_relleno)
            
            if estilo.attrib.get('bgcolor', None):
                if not patron_relleno:
                    pattern.append('pattern solid')
                    
                pattern.append('fore-color %s' % estilo.attrib['bgcolor'])
                
            if pattern:
                style.append('pattern: %s' % (','.join(pattern)))
                
            border = []
            if estilo.find('border') != None:
                borde = estilo.find('border')
                
                # color
                color_borde = ''
                if borde.attrib.get('color', None):
                    color_borde = borde.attrib['color']
                
                # line
                linea_borde = 'thin'
                if borde.attrib.get('line', None):
                    linea_borde = borde.attrib['line']
                
                # top
                top_color = ''
                top_line = ''
                if borde.find('top') != None:
                    tp = borde.find('top')
                    if tp.attrib.get('color', None):
                        top_color = tp.attrib['color']
                        
                    elif color_borde:
                        top_color = color_borde
                        
                    if tp.attrib.get('line', None):
                        top_line = tp.attrib['line']
                    
                    elif linea_borde:
                        top_line = linea_borde
                        
                if top_color:
                    border.append('top_color %s' % top_color)
                    
                if top_line:
                    border.append('top %s' % top_line)
                
                # bottom
                bottom_color = ''
                bottom_line = ''
                if borde.find('bottom') != None:
                    bt = borde.find('bottom')
                    if bt.attrib.get('color', None):
                        bottom_color = bt.attrib['color']
                        
                    elif color_borde:
                        bottom_color = color_borde
                        
                    if bt.attrib.get('line', None):
                        bottom_line = bt.attrib['line']
                    
                    elif linea_borde:
                        bottom_line = linea_borde

                if bottom_color:
                    border.append('bottom_color %s' % bottom_color)
                    
                if bottom_line:
                    border.append('bottom %s' % bottom_line)
                
                # left
                left_color = ''
                left_line = ''
                if borde.find('left') != None:
                    lf = borde.find('left')
                    if lf.attrib.get('color', None):
                        left_color = lf.attrib['color']
                        
                    elif color_borde:
                        left_color = color_borde
                        
                    if lf.attrib.get('line', None):
                        left_line = lf.attrib['line']
                    
                    elif linea_borde:
                        left_line = linea_borde

                if left_color:
                    border.append('left_color %s' % left_color)
                    
                if left_line:
                    border.append('left %s' % left_line)

                # right
                right_color = ''
                right_line = ''
                if borde.find('right') != None:
                    rt = borde.find('right')
                    if rt.attrib.get('color', None):
                        right_color = rt.attrib['color']
                        
                    elif color_borde:
                        right_color = color_borde
                        
                    if rt.attrib.get('line', None):
                        right_line = rt.attrib['line']
                    
                    elif linea_borde:
                        right_line = linea_borde

                if right_color:
                    border.append('right_color %s' % right_color)
                    
                if right_line:
                    border.append('right %s' % right_line)

                if border:
                    style.append('border: %s' % (','.join(border)))
                    
        return style
    
    def create(self, params, filename=None):
        """
        IN
          params    <dict>
          filename  <str> (opcional)
          
        OUT
          El XML del informe
          <str>
        """
        
        util = self.Util()
        self.tmpl.globals['util'] = util
        self.tmpl.globals['xlrd'] = xlrd

        informe_xml = self.tmpl.render(**params).encode('utf-8')
        
        root = etree.fromstring(informe_xml)
        
        wb = Workbook()
        
        estilos = {}
        
        xfs0 = XFStyle()
        
        n_sheet = 0
        for sheet in root.iter('sheet'):
            
            n_sheet += 1
            
            # title
            title = sheet.attrib.get('title', 'Sheet-%d' % n_sheet)
                
            ws = wb.add_sheet(title, True)
            
            util.init_count()
            for item in sheet:
                
                # style
                if item.tag == 'style':
                    estilos[item.attrib['name']] = easyxf(';'.join(self.calcular_estilo(item)))
                    
                # image
                if item.tag == 'image':
                    bm_path = item.find('path').text
                    col = int(item.attrib.get('col', 0))
                    scale_x = float(item.attrib.get('scale_x', '1'))
                    scale_y = float(item.attrib.get('scale_y', '1'))
                    ws.insert_bitmap(bm_path, util.current_line(), col, scale_x=scale_x, scale_y=scale_y) 
                
                # cell
                if item.tag == 'cell':
                    
                    col = int(item.attrib['col'])
                    
                    num_format = None
                    
                    # width
                    width = item.attrib.get('width')
                    
                    # value
                    value = item.find('value')
                    dato = value.text
                    # type: aplicar una conversión (str, int, float, ...)
                    type_ = item.attrib.get('type')
                    if type_:
                        tipo = type_.split(';')
                        
                        # date
                        if tipo[0] == 'date':
                            try:
                                dato = strtodate(dato, fmt='%Y-%m-%d')
                            except:
                                dato = None
    
                        # time
                        elif tipo[0] == 'time':
                            try:
                                dato = strtotime(dato)
                            except:
                                dato = None
                        
                        # formula
                        elif tipo[0] == 'formula':
                            dato = Formula(dato)
                        
                        # resto de tipos
                        else:
                            if dato:
                                try:
                                    f = lambda t,v: t(v)
                                    dato = f(eval(tipo[0]), dato)
                                except:
                                    dato = 'ERROR! (%s)' % dato
                        
                        # date;dd/mm/yyyy
                        # date;dd/mmm/yyyy
                        # date;NN D MMMM YYYY
                        # time;hh:mm
                        # float;#,##0.00
                        # float;+#,##0.00
                        # float;#,##0.00;[RED]-#,##0.00
                        if len(tipo) > 1:
                            num_format = ';'.join(tipo[1:])
                            
                        #print dato, tipo, num_format
    
                    # style: aplicar un estilo
                    estilo = item.find('style')
                    xfs = xfs0
                    if estilo != None:
                        if not estilos.has_key(estilo.attrib['name']):
                            xfs = easyxf(';'.join(self.calcular_estilo(estilo))) 
        
                            estilos[estilo.attrib['name']] = xfs
                            
                        else:
                            xfs = estilos[estilo.attrib['name']]
                        
                    xfs.num_format_str = 'General'    
                    if num_format:
                        if not xfs:
                            xfs = XFStyle()

                        xfs.num_format_str = num_format
                        
                    # merge
                    if item.attrib.has_key('merge') and item.attrib['merge']:
                        try:
                            _merge = re.search(r'(\d+)\s+(\d+)', item.attrib['merge'], re.U)
                            if _merge:
                                hg = int(_merge.group(1))
                                wd = int(_merge.group(2))
                            
                                _current_line = util.current_line()
                                ws.write_merge(_current_line, _current_line + hg - 1,
                                               col, col + wd -1, 
                                               dato, xfs)
                            else:
                                raise Exception(u'merge: Invalid format')
                        
                        except Exception:
                            ws.write(util.current_line(), col, dato, xfs)
                            
                    else:
                        ws.write(util.current_line(), col, dato, xfs)
                        
                    if width:
                        ws.col(col).width = int(width)*256
                
                elif item.tag == 'line_feed':
                    n = int(item.attrib.get('n', 1))
                    util.line_feed(n)
                    
                elif item.tag == 'bookmark':
                    util.add_bookmark(item.attrib['name'])
                
        if filename:
            wb.save(filename)
        
        return informe_xml